from ...types import ExternalAsset


class DomoAsset(ExternalAsset):
    """Domo assets"""

    AUDIT = "audit"
    DATAFLOWS = "dataflows"
    DATASETS = "datasets"
    PAGES = "pages"
    USERS = "users"
