from ...types import ExternalAsset


class ThoughtspotAsset(ExternalAsset):
    """Thoughtspot assets"""

    ANSWERS = "answers"
    ANSWER_USAGES = "answer_usages"
    LIVEBOARDS = "liveboards"
    LIVEBOARD_USAGES = "liveboard_usages"
    LOGICAL_TABLES = "logical_tables"
