from ...types import ExternalAsset


class SigmaAsset(ExternalAsset):
    """Sigma assets"""

    DATAMODELS = "datamodels"
    DATASETS = "datasets"
    DATASET_SOURCES = "dataset_sources"
    ELEMENTS = "elements"
    FILES = "files"
    LINEAGES = "lineages"
    MEMBERS = "members"
    QUERIES = "queries"
    WORKBOOKS = "workbooks"
    WORKBOOK_SOURCES = "workbook_sources"
